@@ -30,7 +30,6 @@
 #include <sys/wait.h>
 #include <sys/poll.h>
 #include <string.h>
-#include <dlfcn.h>
 
 #include <glib.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
@@ -68,6 +67,8 @@
 #error Unknown plugin type
 #endif
 
+#define DASHES "--"
+
 /* How much data bytes to request */
 #define PLUGIN_STREAM_CHUNK_SIZE (8 * 1024)
 
@@ -119,9 +120,6 @@ static void
 cb_stop_sending_data (DBusGProxy  *proxy,
 		      totemPlugin *plugin)
 {
-	if (!plugin->stream)
-		return;
-
 	D("Stop sending data signal received");
 	if (CallNPN_DestroyStreamProc (mozilla_functions.destroystream,
 				plugin->instance,
@@ -155,6 +153,22 @@ get_real_mimetype (const char *mimetype)
 	return NULL;
 }
 
+static gboolean
+is_mimetype_supported (const char *mimetype)
+{
+	const totemPluginMimeEntry *mimetypes;
+	PRUint32 count;
+
+	totemScriptablePlugin::PluginMimeTypes (&mimetypes, &count);
+
+	for (PRUint32 i = 0; i < count; ++i) {
+		if (strcmp (mimetypes[i].mimetype, mimetype) == 0)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 static gboolean
 is_supported_scheme (const char *url)
 {
@@ -188,6 +202,7 @@ totem_plugin_fork (totemPlugin *plugin)
 
 	arr = g_ptr_array_new ();
 
+#ifdef TOTEM_RUN_IN_SOURCE_TREE
 	if (g_file_test ("./totem-mozilla-viewer",
 				G_FILE_TEST_EXISTS) != FALSE) {
 		g_ptr_array_add (arr, g_strdup ("./totem-mozilla-viewer"));
@@ -195,6 +210,11 @@ totem_plugin_fork (totemPlugin *plugin)
 		g_ptr_array_add (arr,
 				g_strdup (LIBEXECDIR"/totem-mozilla-viewer"));
 	}
+#else
+	g_ptr_array_add (arr,
+			g_strdup (LIBEXECDIR"/totem-mozilla-viewer"));
+#endif
+
 
 	/* For the RealAudio streams */
 	if (plugin->width == 0 && plugin->height == 0) {
@@ -203,58 +223,58 @@ totem_plugin_fork (totemPlugin *plugin)
 	}
 
 	if (plugin->window) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_XID));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_XID));
 		g_ptr_array_add (arr, g_strdup_printf ("%lu", plugin->window));
 	}
 
 	if (plugin->width > 0) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_WIDTH));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_WIDTH));
 		g_ptr_array_add (arr, g_strdup_printf ("%d", plugin->width));
 	}
 
 	if (plugin->height > 0) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_HEIGHT));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_HEIGHT));
 		g_ptr_array_add (arr, g_strdup_printf ("%d", plugin->height));
 	}
 
 	if (plugin->src) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_URL));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_URL));
 		g_ptr_array_add (arr, g_strdup (plugin->src));
 	}
 
 	if (plugin->href) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_HREF));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_HREF));
 		g_ptr_array_add (arr, g_strdup (plugin->href));
 	}
 
 	if (plugin->target) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_TARGET));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_TARGET));
 		g_ptr_array_add (arr, g_strdup (plugin->target));
 	}
 
 	if (plugin->mimetype) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_MIMETYPE));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_MIMETYPE));
 		g_ptr_array_add (arr, g_strdup (plugin->mimetype));
 	}
 
 	if (plugin->controller_hidden) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_CONTROLS_HIDDEN));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_CONTROLS_HIDDEN));
 	}
 
 	if (plugin->hidden) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_HIDDEN));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_HIDDEN));
 	}
 
  	if (plugin->repeat) {
- 		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_REPEAT));
+ 		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_REPEAT));
  	}
 
 	if (plugin->noautostart) {
-		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_NOAUTOSTART));
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_NOAUTOSTART));
 	}
  
  	if (plugin->is_playlist) {
- 		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_PLAYLIST));
+ 		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_PLAYLIST));
  		g_ptr_array_add (arr, g_strdup (plugin->local));
  	} else {
 		/* plugin->local is only TRUE for playlists */
@@ -400,6 +420,11 @@ totem_plugin_new_instance (NPMIMEType mimetype,
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
 
+	/* Make sure the plugin stays resident to avoid crashers when 
+	 * reloading the GObject types */
+	mozilla_functions.setvalue (instance,
+			NPPVpluginKeepLibraryInMemory, (void *)TRUE);
+
 	instance->pdata = mozilla_functions.memalloc(sizeof(totemPlugin));
 	plugin = (totemPlugin *) instance->pdata;
 
@@ -484,6 +509,20 @@ totem_plugin_new_instance (NPMIMEType mimetype,
 				plugin->src = resolve_relative_uri (docURI, argv[i]);
 				need_req = TRUE;
 			}
+		} else if (g_ascii_strcasecmp (argn[i], "qtsrc") == 0) {
+			/* Quicktime parameter documented in
+			 * http://developer.apple.com/documentation/QuickTime/QT6WhatsNew/Chap1/chapter_1_section_13.html */
+			if (plugin->src != NULL)
+				g_free (plugin->src);
+			if (plugin->stream != nsnull) {
+				CallNPN_DestroyStreamProc
+					(mozilla_functions.destroystream,
+					 plugin->instance, plugin->stream,
+					 NPRES_DONE);
+				plugin->stream = nsnull;
+			}
+			plugin->src = resolve_relative_uri (docURI, argv[i]);
+			need_req = TRUE;
 		} else if (g_ascii_strcasecmp (argn[i], "href") == 0) {
 			plugin->href = resolve_relative_uri (docURI, argv[i]);
 		} else if (g_ascii_strcasecmp (argn[i], "cache") == 0) {
@@ -652,6 +691,9 @@ totem_plugin_new_stream (NPP instance,
 
 	D("plugin_new_stream type: %s url: %s", type, plugin->src);
 
+	if (is_mimetype_supported (type) == FALSE)
+		return NPERR_INVALID_PLUGIN_ERROR;
+
 	//FIXME need to find better semantics?
 	//what about saving the state, do we get confused?
 	if (g_str_has_prefix (plugin->src, "file://")) {
@@ -731,11 +773,6 @@ static int32 totem_plugin_write (NPP instance, NPStream *stream, int32 offset,
 	if (plugin == NULL)
 		return -1;
 
-	/* We already know it's a playlist, don't try to check it again
-	 * and just wait for it to be on-disk */
-	if (plugin->is_playlist != FALSE)
-		return len;
-
 	if (!plugin->player_pid) {
 		if (!plugin->stream) {
 			g_warning ("No stream in NPP_Write!?");
@@ -831,13 +868,6 @@ totem_plugin_url_notify (NPP instance, const char* url,
 	D("plugin_url_notify");
 }
 
-static void
-totem_plugin_print (NPP instance,
-                    NPPrint* platformPrint)
-{
-	D("plugin_print");
-}
-
 static char *
 totem_plugin_get_description (void)
 {
@@ -990,17 +1020,6 @@ NP_Initialize (NPNetscapeFuncs * moz_funcs,
 	if (plugin_funcs->size < sizeof (NPPluginFuncs))
 		return NPERR_INVALID_FUNCTABLE_ERROR;
 
-	/* we want to open libdbus-glib-1.so.2 in such a way
-	 * in such a way that it becomes permanentely resident */
-	void *handle;
-	handle = dlopen ("libdbus-glib-1.so.2", RTLD_NOW | RTLD_NODELETE);
-	if (!handle) {
-		fprintf (stderr, "%s\n", dlerror()); 
-		return NPERR_MODULE_LOAD_FAILED_ERROR;
-	}
-	/* RTLD_NODELETE allows us to close right away ... */
-	dlclose(handle);
-
 	/*
 	 * Copy all of the fields of the Mozilla function table into our
 	 * copy so we can call back into Mozilla later.  Note that we need
@@ -1055,7 +1074,7 @@ NP_Initialize (NPNetscapeFuncs * moz_funcs,
 		NewNPP_WriteReadyProc(totem_plugin_write_ready);
 	plugin_funcs->write = NewNPP_WriteProc(totem_plugin_write);
 	/* Printing ? */
-	plugin_funcs->print = NewNPP_PrintProc(totem_plugin_print);
+	plugin_funcs->print = NewNPP_PrintProc(NULL);
 	/* What's that for ? */
 	plugin_funcs->event = NewNPP_HandleEventProc(NULL);
 	plugin_funcs->urlnotify =
