@@ -586,7 +586,7 @@ bacon_volume_scale_value_changed (GtkRange * range)
   GdkPixbuf *buf;
 
   if (val == adj->lower)
-    s = "stock_volume-0";
+    s = "stock_volume-mute";
   else if (val > adj->lower && val <= adj->lower + step)
     s = "stock_volume-min";
   else if (val > adj->lower + step && val <= adj->lower + step * 2)
@@ -598,6 +598,7 @@ bacon_volume_scale_value_changed (GtkRange * range)
   gtk_icon_size_lookup (button->size, &w, &h);
   buf = gtk_icon_theme_load_icon (button->theme, s, w, 0, NULL);
   gtk_image_set_from_pixbuf (GTK_IMAGE (button->image), buf);
+  g_object_unref (buf);
 #endif
 
   /* signal */
