@@ -80,106 +80,69 @@ typedef struct _CdCache {
  * Resolve relative paths
  */
 
-/* Copied from gtk+/gtk/gtkfilesystemunix.c */
-
-/* If this was a publically exported function, it should return
- * a dup'ed result, but we make it modify-in-place for efficiency
- * here, and because it works for us.
- */
-static void
-canonicalize_filename (gchar *filename)
+static char *
+totem_disc_resolve_link (const char *dev, const char *buf)
 {
-  gchar *p, *q;
-  gboolean last_was_slash = FALSE;
+  char *parent, *new, *result;
 
-  p = filename;
-  q = filename;
+  /* is it an absolute path? */
+  if (g_path_is_absolute (buf) != FALSE) {
+    result = realpath (buf, NULL);
+    if (result == NULL)
+      result = g_strdup (buf);
+  } else {
+    parent = g_path_get_dirname (dev);
+    new = g_build_filename (parent, buf, NULL);
+    result = realpath (new, NULL);
+    if (result == NULL)
+      result = g_strdup (new);
+    g_free (new);
+    g_free (parent);
+  }
 
-  while (*p)
-    {
-      if (*p == G_DIR_SEPARATOR)
-	{
-	  if (!last_was_slash)
-	    *q++ = G_DIR_SEPARATOR;
-
-	  last_was_slash = TRUE;
-	}
-      else
-	{
-	  if (last_was_slash && *p == '.')
-	    {
-	      if (*(p + 1) == G_DIR_SEPARATOR ||
-		  *(p + 1) == '\0')
-		{
-		  if (*(p + 1) == '\0')
-		    break;
-
-		  p += 1;
-		}
-	      else if (*(p + 1) == '.' &&
-		       (*(p + 2) == G_DIR_SEPARATOR ||
-			*(p + 2) == '\0'))
-		{
-		  if (q > filename + 1)
-		    {
-		      q--;
-		      while (q > filename + 1 &&
-			     *(q - 1) != G_DIR_SEPARATOR)
-			q--;
-		    }
-
-		  if (*(p + 2) == '\0')
-		    break;
-
-		  p += 2;
-		}
-	      else
-		{
-		  *q++ = *p;
-		  last_was_slash = FALSE;
-		}
-	    }
-	  else
-	    {
-	      *q++ = *p;
-	      last_was_slash = FALSE;
-	    }
-	}
-
-      p++;
-    }
-
-  if (q > filename + 1 && *(q - 1) == G_DIR_SEPARATOR)
-    q--;
-
-  *q = '\0';
+  return result;
 }
 
-static char *
-totem_resolve_symlink (const char *device, GError **error)
-{
-  char *dir, *link;
-  char *f;
-  char *f1;
+/*
+ * So, devices can be symlinks and that screws up.
+ */
 
-  f = g_strdup (device);
-  while (g_file_test (f, G_FILE_TEST_IS_SYMLINK)) {
-    link = g_file_read_link (f, error);
-    if(link == NULL) {
-      g_free (f);
+static char *
+get_device (const char *device,
+	    GError     **error)
+{
+  char *buf;
+  char *dev = g_strdup (device);
+  struct stat st;
+
+  while (1) {
+    char *new;
+
+    if (lstat (dev, &st) != 0) {
+      g_set_error (error, 0, 0,
+          _("Failed to find real device node for %s: %s"),
+          dev, g_strerror (errno));
+      g_free (dev);
       return NULL;
     }
 
-    dir = g_path_get_dirname (f);
-    f1 = g_build_filename (dir, link, NULL);
-    g_free (dir);
-    g_free (f);
-    f = f1;
+    if (!S_ISLNK (st.st_mode))
+      break;
+
+    if (!(buf = g_file_read_link (dev, NULL))) {
+      g_set_error (error, 0, 0,
+          _("Failed to read symbolic link %s: %s"),
+          dev, g_strerror (errno));
+      g_free (dev);
+      return NULL;
+    }
+    new = totem_disc_resolve_link (dev, buf);
+    g_free (dev);
+    g_free (buf);
+    dev = new;
   }
 
-  if (f != NULL)
-    canonicalize_filename (f);
-  return f;
+  return dev;
 }
 
 static gboolean
@@ -199,7 +162,7 @@ cd_cache_get_dev_from_volumes (GnomeVFSVolumeMonitor *mon, const char *device,
     volume = list->data;
     if (!(pdev = gnome_vfs_volume_get_device_path (volume)))
       continue;
-    pdev2 = totem_resolve_symlink (pdev, NULL);
+    pdev2 = get_device (pdev, NULL);
     if (!pdev2) {
       g_free (pdev);
       continue;
@@ -250,7 +213,7 @@ cd_cache_get_dev_from_drives (GnomeVFSVolumeMonitor *mon, const char *device,
     drive = list->data;
     if (!(pdev = gnome_vfs_drive_get_device_path (drive)))
       continue;
-    pdev2 = totem_resolve_symlink (pdev, NULL);
+    pdev2 = get_device (pdev, NULL);
     if (!pdev2) {
       g_free (pdev);
       continue;
@@ -305,12 +268,8 @@ cd_cache_new_hal_ctx (void)
     return NULL;
 
   dbus_error_init (&error);
-#if (DBUS_VERSION_MAJOR == 0) && (DBUS_VERSION_MINOR < 60)
-  conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
-  g_warning ("might crash, see http://bugzilla.gnome.org/show_bug.cgi?id=345203");
-#else
   conn = dbus_bus_get_private (DBUS_BUS_SYSTEM, &error);
-#endif
+
   if (conn != NULL && !dbus_error_is_set (&error)) {
     if (!libhal_ctx_set_dbus_connection (ctx, conn)) {
       libhal_ctx_free (ctx);
@@ -362,7 +321,7 @@ cd_cache_new (const char *dev,
   }
 
   /* retrieve mountpoint from gnome-vfs volumes and drives */
-  device = totem_resolve_symlink (local, error);
+  device = get_device (local, error);
   g_free (local);
   if (!device)
     return NULL;
@@ -609,12 +568,7 @@ cd_cache_free (CdCache *cache)
     conn = libhal_ctx_get_dbus_connection (cache->ctx);
     libhal_ctx_shutdown (cache->ctx, NULL);
     libhal_ctx_free(cache->ctx);
-#if (DBUS_VERSION_MAJOR == 0) && (DBUS_VERSION_MINOR < 60)
-    dbus_connection_unref (conn);
-#else
     dbus_connection_close (conn);
-    dbus_connection_unref (conn);
-#endif
 
     g_free (cache->disc_udi);
 #elif HAVE_HAL_0_2
