@@ -18,13 +18,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#define TOTEM_OPTION_XID		"--xid"
-#define TOTEM_OPTION_WIDTH		"--width"
-#define TOTEM_OPTION_HEIGHT		"--height"
-#define TOTEM_OPTION_URL		"--url"
-#define TOTEM_OPTION_CONTROLS_HIDDEN	"--nocontrols"
-#define TOTEM_OPTION_HREF		"--href"
-#define TOTEM_OPTION_TARGET		"--target"
-#define TOTEM_OPTION_HIDDEN		"--hidden"
-#define TOTEM_OPTION_MIMETYPE		"--mimetype"
-
+#define TOTEM_OPTION_XID		"xid"
+#define TOTEM_OPTION_WIDTH		"width"
+#define TOTEM_OPTION_HEIGHT		"height"
+#define TOTEM_OPTION_URL		"url"
+#define TOTEM_OPTION_CONTROLS_HIDDEN	"nocontrols"
+#define TOTEM_OPTION_HREF		"href"
+#define TOTEM_OPTION_TARGET		"target"
+#define TOTEM_OPTION_HIDDEN		"hidden"
+#define TOTEM_OPTION_MIMETYPE		"mimetype"
+#define TOTEM_OPTION_PLAYLIST		"playlist"
+#define TOTEM_OPTION_REPEAT		"repeat"
+#define TOTEM_OPTION_NOAUTOSTART	"noautostart"
